function ad612(command,p2)
%AD612 Hardware driver setup for the AD612 board.
%
%   This function creates the GUI for setting device driver parameters 
%   for the AD612 board.

%	Version 3.11  10-January-2003
%	Copyright (c) 1992-2003   HUMUSOFT s.r.o.

boardname=rttool('Load','ad612');
figh=findobj('Type','figure','Name',boardname);

if isempty(figh)

% Draw the GUI figure

figh=dialog('HandleVisibility','on','Visible','off','Name',boardname,'Tag','RTDRIVER_Window',...
            'Position',[100 80 285 215],'KeyPressFcn',...
            'ad612 @key');

% Define PCI Slot control

uicontrol('Style','text','Position',[20 183 115 20],'HorizontalAlignment','Left',...
          'String','PCI Slot :');
uicontrol('Style','edit','BackGroundColor',[1 1 1],'Position',[75 185 40 20],...
          'HorizontalAlignment','Left','CallBack','ad612 @cline;','Tag','PCISlot');

uicontrol('Style','check','Position',[140 185 90 20],...
          'String',' Auto-detect','CallBack','ad612 @autoslot;ad612 @cline;','Tag','Auto_Slot');

% Define Input format control

uicontrol('Style','text','Position',[30 127 90 20],'HorizontalAlignment','Left',...
          'String','Digital Input:');
uicontrol('Style','radio','Position',[30 110 45 20],'String','Bit','Min',-1,'Max',0,...
          'CallBack','rbutton;ad612 @cline;','Tag','Input_Format');
uicontrol('Style','radio','Position',[80 110 60 20],'String','Byte','Max',2,...
          'CallBack','rbutton;ad612 @cline;','Tag','Input_Format');
  
% Define Output format control

uicontrol('Style','text','Position',[170 127 90 20],'HorizontalAlignment','Left',...
          'String','Digital Output:');
uicontrol('Style','radio','Position',[170 110 45 20],'String','Bit','Min',-1,'Max',0,...
          'CallBack','rbutton;ad612 @cline;','Tag','Output_Format');
uicontrol('Style','radio','Position',[220 110 60 20],'String','Byte','Max',3,...
          'CallBack','rbutton;ad612 @cline;','Tag','Output_Format');

% Define Gain control

uicontrol('Style','push','Position',[190 75 85 25],'String','Gains ...',...
          'CallBack','ad612 @gain','Tag','Gain_Button');

% Define Command line control

uicontrol('Style','text','Position',[10 65 100 16],'HorizontalAlignment','Left',...
          'String','Command line:');
uicontrol('Style','text','BackGroundColor',[1 1 1],'HorizontalAlignment','Left',...
          'Position',[10 45 265 20],'Tag','Command_Line');

% Define buttons

uicontrol('Style','push','Position',[19 10 49 25],'String','OK',...
          'CallBack','ad612 @ok');
uicontrol('Style','push','Position',[85 10 49 25],'String','Reset',...
          'CallBack','ad612 @default');
uicontrol('Style','push','Position',[151 10 49 25],'String','Save',...
          'CallBack','ad612 @cline @save');
uicontrol('Style','push','Position',[217 10 49 25],'String','Cancel',...
          'CallBack','close');
  
% Draw figure

set(findobj(figh,'Units','pixels'),'Units','normalized');
if nargin<2
  drvs=rttool('Read',0,'Drivers');
  drvf=0;
  for i=drvs
    if strcmp(rttool('Read',i,'Name'),'ad612')
      drvf=i;
      break;
    end;
  end;
  if drvf
    [addr,opt]=rttool('Read',drvf,'Address');
    ad612(addr,opt);
  else
    ad612 @default;
  end;
else
  ad612(command,p2);
end;
set(figh,'Visible','on');
drawnow;

else

% Bring the window to front if already open

  if nargin==0
    figure(figh);

% Set default parameters to GUI window

  elseif ~isstr(command)
    if (command==65535)
      set(findobj(figh,'Tag','Auto_Slot'),'Value',1)
      set(findobj(figh,'Tag','PCISlot'),'String','0');
      ad612 @autoslot;
    else
      drvloaded=rttool('Read',0,'Drivers');
      for i=drvloaded
        if strcmp(rttool('Read',i,'Name'), 'ad612')
          set(findobj(figh,'Tag','Auto_Slot'),'Enable','off')
          break;
        end;
      end;
      set(findobj(figh,'Tag','PCISlot'),'String',command);
    end;
    butt=findobj(figh,'Tag','Input_Format');
    bmax=get(butt,'Max');
    bmin=get(butt,'Min');
    bmax=[bmax{:}]-[bmin{:}];
    rbutton(butt(find(bmax==rem(p2(1),2)+1)));
    butt=findobj(figh,'Tag','Output_Format');
    bmax=get(butt,'Max');
    bmin=get(butt,'Min');
    bmax=[bmax{:}]-[bmin{:}];
    rbutton(butt(find(bmax==rem(p2(1),4)-rem(p2(1),2)+1)));
    gbv=zeros(4,8);
    gains=[p2(2:size(p2,2)) zeros(1,9-size(p2,2))];
    for i=1:8
      gbv(find([0:3]==gains(i)),i)=1;
    end;
    set(findobj(figh,'Tag','Gain_Button'),'UserData',gbv);
    ad612 @cline;

% Call if Autodetect changed

  elseif strcmp(command,'@autoslot')
    if get(findobj(figh,'Tag','Auto_Slot'),'Value')
      set(findobj(figh,'Tag','PCISlot'), 'Value', 65535, 'Enable','off');
    else
      set(findobj(figh,'Tag','PCISlot'), 'Value', 0, 'Enable','on');
    end;
   

% Call if Gain pressed

  elseif strcmp(command,'@gain')
    gbv=get(findobj(figh,'Tag','Gain_Button'),'UserData');
    gbh=zeros(size(gbv));
    chfh=dialog('Visible','off',...
                'Name','Select Input Gains',...
                'Position',[102 138 280 98]);
    for i=1:8
      uicontrol('Style','text','Position',[70+15*i 81 9 12],...
                'String',rem(i,10));
      for j=1:4
        gbh(j,i)=uicontrol('Style','radio','Position',[67+15*i 80-18*j 15 15],...
                           'Value',gbv(j,i),'CallBack','rbutton',...
                           'Tag',sprintf('Gain_%d',i));
      end;
    end;

    gains=str2mat('0-5 V','5 V','0-10 V','10 V');
    for j=1:4
      uicontrol('Style','push','Position',[7 79-18*j 70 18],...
                'String',[gains(j,:) ' >>'],...
                'CallBack',sprintf('ad612(''@gainbutt'',%d)',j));
    end;
    uicontrol('Style','push','Position',[220 45 50 25],...
              'String','OK',...
              'CallBack','ad612 @gainok');
    uicontrol('Style','push','Position',[220 10 50 25],...
              'String','Cancel',...
              'CallBack','close');

    set(chfh,'UserData',gbh,'Visible','on');

% Call if Gain OK pressed

  elseif strcmp(command,'@gainok')
    gbh=get(gcf,'UserData');
    gbv=get(gbh,'Value');
    set(findobj(figh,'Tag','Gain_Button'), 'UserData', reshape([gbv{:}],size(gbh)));
    close;
    ad612 @cline;

% Call if Gain button pressed

  elseif strcmp(command,'@gainbutt')
    gbh=get(gcf,'UserData');
    for i=gbh(p2,:)
      rbutton(i);
    end;

% Call if Command line to be updated

  elseif strcmp(command,'@cline')
    addr=max([get(findobj(figh,'Tag','PCISlot'),'Value') str2num(get(findobj(figh,'Tag','PCISlot'),'String'))]);
    ioform = get([findobj(figh,'Tag','Input_Format')' findobj(figh,'Tag','Output_Format')'], 'Value');
    ioform = sum([ioform{:}]);
    og=[0:3]*get(findobj(figh,'Tag','Gain_Button'),'UserData');
    cl=sprintf('rtload(''ad612'',%d,[%d',addr,ioform);
    cl=[cl sprintf(' %d',og)];
    cl=[cl ']);'];
    set(findobj(figh,'Tag','Command_Line'),'String',cl);
    if nargin==2
      opt=str2num(cl(min(find(cl=='[')):max(find(cl==']'))));
      [f p]=uiputfile('ad612.mat','Save driver defaults');
      if f
        save([p f],'addr','opt');
      end;
    end;

% Call if OK pressed

  elseif strcmp(command,'@ok')
    try
      rteval_(get(findobj(figh,'Tag','Command_Line'),'String'));
      close;
    catch
      rterror(lasterr);
    end;

% Call if Reset pressed

  elseif strcmp(command,'@default')
    if exist('ad612.mat')==2
      load ad612;
    else
      [dummy,addr,opt]=rttool('Load','ad612');
    end;
    ad612(addr,opt);

% Call if key pressed

  elseif strcmp(command,'@key')
    ch=get(figh,'CurrentCharacter');
    if ch==27
      close;
    elseif ch==13
      ad612 @ok;
    elseif upper(ch)=='R'
      ad612 @revert;
    elseif upper(ch)=='S'
      ad612 @cline @save;
    elseif upper(ch)=='G'
      ad612 @gain;
    end;

  end

end


